module.exports = {
  plugins: ['@typescript-eslint', 'prettier'],
  extends: [
    'eslint:recommended',
    'plugin:@typescript-eslint/recommended',
    'plugin:@typescript-eslint/recommended-requiring-type-checking',
    'plugin:prettier/recommended',
  ],
  rules: {
    'require-await': 'error',
    'arrow-body-style': ['error', 'as-needed'],
    'prefer-arrow-callback': [
      'error',
      { allowNamedFunctions: false, allowUnboundThis: false },
    ],
    'no-return-await': ['error'],
    curly: ['error', 'multi-or-nest', 'consistent'],
    eqeqeq: ['error', 'smart'],
    'no-throw-literal': ['error'],
    '@typescript-eslint/no-floating-promises': ['warn'],
    '@typescript-eslint/no-misused-promises': [
      'warn',
      {
        checksConditionals: true,
        checksSpreads: true,
        checksVoidReturn: {
          attributes: false,
        },
      },
    ],
    '@typescript-eslint/no-namespace': 'off',
    '@typescript-eslint/no-empty-function': [
      'error',
      {
        allow: [
          'decoratedFunctions',
          'private-constructors',
          'protected-constructors',
          'overrideMethods',
        ],
      },
    ],
    '@typescript-eslint/no-explicit-any': 'off',
    '@typescript-eslint/restrict-template-expressions': [
      'error',
      {
        allowNumber: true,
        allowBoolean: true,
        allowAny: true,
        allowRegExp: true,
        allowNullish: true,
      },
    ],
    '@typescript-eslint/no-unsafe-member-access': 'off',
    '@typescript-eslint/no-empty-interface': [
      'error',
      {
        allowSingleExtends: true,
      },
    ],
    '@typescript-eslint/no-unsafe-argument': 'warn',
    '@typescript-eslint/explicit-member-accessibility': [
      'error',
      {
        accessibility: 'no-public',
        overrides: {
          parameterProperties: 'explicit',
        },
      },
    ],
    '@typescript-eslint/restrict-plus-operands': [
      'error',
      { checkCompoundAssignments: true, allowAny: false },
    ],
    '@typescript-eslint/naming-convention': [
      'warn',
      {
        selector: 'default',
        format: ['camelCase'],
        leadingUnderscore: 'allow',
        trailingUnderscore: 'allow',
      },
      {
        selector: 'variable',
        format: ['camelCase', 'UPPER_CASE'],
        leadingUnderscore: 'allow',
        trailingUnderscore: 'allow',
      },
      {
        selector: 'typeLike',
        format: ['PascalCase'],
      },
      {
        selector: 'variable',
        modifiers: ['const', 'exported'],
        types: ['boolean', 'string', 'array', 'number'],
        format: ['UPPER_CASE', 'PascalCase'],
      },
      // Enforce that boolean variables are prefixed with an allowed verb
      {
        selector: 'variable',
        types: ['boolean'],
        format: ['PascalCase'],
        prefix: ['is', 'should', 'has', 'can', 'did', 'will'],
        leadingUnderscore: 'allow',
      },
      // Enforce that type parameters (generics) are prefixed with `T`
      {
        selector: 'typeParameter',
        format: ['PascalCase'],
        prefix: ['T'],
      },
      // Ignore destructured names
      {
        selector: 'variable',
        modifiers: ['destructured'],
        format: null,
      },
      // Ignore properties that require quotes
      {
        selector: [
          'classProperty',
          'objectLiteralProperty',
          'typeProperty',
          'classMethod',
          'objectLiteralMethod',
          'typeMethod',
          'accessor',
          'enumMember',
        ],
        format: null,
        modifiers: ['requiresQuotes'],
      },
    ],
  },
};
